from django.core.exceptions import ImproperlyConfigured
from django.core.mail import EmailMultiAlternatives


class DjrillMessage(EmailMultiAlternatives):
    alternative_subtype = "mandrill"

    def __init__(self, subject='', body='', from_email=None, to=None, bcc=None,
        connection=None, attachments=None, headers=None, alternatives=None,
        cc=None, from_name=None, tags=None, track_opens=True,
        track_clicks=True):

        super(DjrillMessage, self).__init__(subject, body, from_email, to, bcc,
            connection, attachments, headers, alternatives, cc)

        self.from_name = from_name
        self.tags = self._set_mandrill_tags(tags)
        self.track_opens = track_opens
        self.track_clicks = track_clicks

    def _set_mandrill_tags(self, tags):
        """
        Check that all tags are below 50 chars and that they do not start
        with an underscore.

        Raise ImproperlyConfigured if an underscore tag is passed in to
        alert the user. Any tag over 50 chars is left out of the list.
        """
        tag_list = []

        for tag in tags:
            if len(tag) <= 50 and not tag.startswith("_"):
                tag_list.append(tag)
            elif tag.startswith("_"):
                raise ImproperlyConfigured(
                    "Tags starting with an underscore are reserved for "
                    "internal use and will cause errors with Mandill's API")

        return tag_list



from django import forms
from django.conf import settings
from django.contrib import messages
from django.core.exceptions import ImproperlyConfigured
from django.core.urlresolvers import reverse
from django.http import (HttpResponseForbidden, HttpResponseRedirect)
from django.utils import simplejson as json
from django.views.generic import TemplateView, View

from djrill.forms import CreateSenderForm

try:
    import requests
except ImportError:
    raise ImportError("Install the damn requirements!")


class DjrillAdminMedia(object):
    def _media(self):
        js = ["js/core.js", "js/jquery.min.js", "js/jquery.init.js"]

        return forms.Media(js=["%s%s" % (settings.ADMIN_MEDIA_PREFIX, url)
            for url in js])
    media = property(_media)


class DjrillApiMixin(object):
    """
    Simple Mixin to grab the api info from the settings file.
    """
    def __init__(self, *args, **kwargs):
        self.api_key = getattr(settings, "MANDRILL_API_KEY", None)
        self.api_url = getattr(settings, "MANDRILL_API_URL", None)

        if not self.api_key:
            raise ImproperlyConfigured("You have not set your mandrill api key "
                "in the settings file.")
        if not self.api_url:
            raise ImproperlyConfigured("You have not added the Mandrill api "
                "url to your settings.py")

    def get_context_data(self, **kwargs):
        kwargs = super(DjrillApiMixin, self).get_context_data(**kwargs)

        status = False
        req = requests.post("%s/%s" % (self.api_url, "users/ping.json"),
            data={"key": self.api_key})
        if req.status_code == 200:
            status = True

        kwargs.update({"status": status})
        return kwargs


class DjrillApiJsonObjectsMixin(object):
    """
    Mixin to grab json objects from the api.
    """
    api_uri = None

    def get_api_uri(self):
        if self.api_uri is None:
            raise ImproperlyConfigured(u"%(cls)s is missing an api_uri. Define "
                u"%(cls)s.api_uri or override %(cls)s.get_api_uri()." % {
                    "cls": self.__class__.__name__
                })

    def get_json_objects(self, extra_dict=None, extra_api_uri=None):
        request_dict = {"key": self.api_key}
        if extra_dict:
            request_dict.update(extra_dict)
        payload = json.dumps(request_dict)
        api_uri = extra_api_uri or self.api_uri
        req = requests.post("%s/%s" % (self.api_url, api_uri),
            data=payload)
        if req.status_code == 200:
            return req.content
        messages.error(self.request, self._api_error_handler(req))
        return json.dumps("error")

    def _api_error_handler(self, req):
        """
        If the API returns an error, display it to the user.
        """
        content = json.loads(req.content)
        return "Mandrill returned a %d response: %s" % (req.status_code,
            content["message"])


class DjrillIndexView(DjrillApiMixin, TemplateView):
    template_name = "djrill/status.html"

    def get(self, request):

        payload = json.dumps({"key": self.api_key})
        req = requests.post("%s/users/info.json" % self.api_url, data=payload)

        return self.render_to_response({"status": json.loads(req.content)})


class DjrillSendersListView(DjrillAdminMedia, DjrillApiMixin,
    DjrillApiJsonObjectsMixin, TemplateView):

    api_uri = "users/senders.json"
    template_name = "djrill/senders_list.html"

    def get(self, request):
        form = CreateSenderForm()
        objects = self.get_json_objects()
        context = self.get_context_data()
        context.update({
            "objects": json.loads(objects),
            "media": self.media,
            "form": form
        })

        return self.render_to_response(context)


class DjrillSenderView(DjrillApiMixin, View):
    api_action = None
    error_message = None
    success_message = None

    def post(self, request):
        email = request.POST.get("email", None)

        if email:
            payload = {
                "key": self.api_key,
                "email": email
            }
            req = requests.post("%s/%s" % (self.api_url, self.api_action),
                data=json.dumps(payload))

            if req.status_code == 200:
                messages.success(request, self.success_message)
            else:
                messages.error(request, self.error_message)
            return HttpResponseRedirect(reverse("admin:djrill_senders"))

        return HttpResponseForbidden()


class DjrillDisableSenderView(DjrillSenderView):
    api_action = "users/disable-sender.json"
    error_message = "Sender was not disabled."
    success_message = "Sender was disabled."


class DjrillVerifySenderView(DjrillSenderView):
    api_action = "users/verify-sender.json"
    error_message = "Sender was not verified."
    success_message = "Sender was verified."


class DjrillAddSenderView(DjrillVerifySenderView):
    error_message = "Sender was not added."
    success_message = "Sender was added."


class DjrillTagListView(DjrillAdminMedia, DjrillApiMixin,
    DjrillApiJsonObjectsMixin, TemplateView):

    api_uri = "tags/list.json"
    template_name = "djrill/tags_list.html"

    def get(self, request):
        objects = self.get_json_objects()
        context = self.get_context_data()
        context.update({
            "objects": json.loads(objects),
            "media": self.media,
        })
        return self.render_to_response(context)

class DjrillUrlListView(DjrillAdminMedia, DjrillApiMixin,
    DjrillApiJsonObjectsMixin, TemplateView):

    api_uri = "urls/list.json"
    template_name = "djrill/urls_list.html"

    def get(self, request):
        objects = self.get_json_objects()
        context = self.get_context_data()
        context.update({
            "objects": json.loads(objects),
            "media": self.media
        })
        return self.render_to_response(context)



from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.test import TestCase

from djrill.mail import DjrillMessage


class DjrillMessageTests(TestCase):
    def setUp(self):
        self.subject = "Djrill baby djrill."
        self.from_name = "Tarzan"
        self.from_email = "test@example"
        self.to = ["King Kong <kingkong@example.com>",
            "Cheetah <cheetah@example.com", "bubbles@example.com"]
        self.text_content = "Wonderful fallback text content."
        self.html_content = "<h1>That's a nice HTML email right there.</h1>"
        self.headers = {"Reply-To": "tarzan@example.com"}
        self.tags = ["track", "this"]

    def test_djrill_message_success(self):
        msg = DjrillMessage(self.subject, self.text_content, self.from_email,
            self.to, tags=self.tags, headers=self.headers,
            from_name=self.from_name)

        self.assertIsInstance(msg, DjrillMessage)
        self.assertEqual(msg.body, self.text_content)
        self.assertEqual(msg.recipients(), self.to)
        self.assertEqual(msg.tags, self.tags)
        self.assertEqual(msg.extra_headers, self.headers)
        self.assertEqual(msg.from_name, self.from_name)

    def test_djrill_message_html_success(self):
        msg = DjrillMessage(self.subject, self.text_content, self.from_email,
            self.to, tags=self.tags)
        msg.attach_alternative(self.html_content, "text/html")

        self.assertEqual(msg.alternatives[0][0], self.html_content)

    def test_djrill_message_tag_failure(self):
        with self.assertRaises(ImproperlyConfigured):
            DjrillMessage(self.subject, self.text_content, self.from_email,
                self.to, tags=["_fail"])

    def test_djrill_message_tag_skip(self):
        """
        Test that tags over 50 chars are not included in the tags list.
        """
        tags = ["works", "awesomesauce",
         "iwilltestmycodeiwilltestmycodeiwilltestmycodeiwilltestmycode"]
        msg = DjrillMessage(self.subject, self.text_content, self.from_email,
            self.to, tags=tags)

        self.assertIsInstance(msg, DjrillMessage)
        self.assertIn(tags[0], msg.tags)
        self.assertIn(tags[1], msg.tags)
        self.assertNotIn(tags[2], msg.tags)



from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.core.mail.backends.base import BaseEmailBackend
from django.core.mail.message import sanitize_address
from django.utils import simplejson as json

import requests


class DjrillBackend(BaseEmailBackend):
    """
    Mandrill API Email Backend
    """

    def __init__(self, fail_silently=False, **kwargs):
        """
        Set the API key, API url and set the action url.
        """
        super(DjrillBackend, self).__init__(**kwargs)
        self.api_key = getattr(settings, "MANDRILL_API_KEY", None)
        self.api_url = getattr(settings, "MANDRILL_API_URL", None)
        self.connection = None

        if not self.api_key:
            raise ImproperlyConfigured("You have not set your mandrill api key "
                "in the settings.py file.")
        if not self.api_url:
            raise ImproperlyConfigured("You have not added the Mandrill api "
                "url to your settings.py")

        self.api_action = self.api_url + "/messages/send.json"
        self.api_verify = self.api_url + "/users/verify-sender.json"

    def open(self, sender):
        """
        """
        self.connection = None

        valid_sender = requests.post(
            self.api_verify, data={"key": self.api_key, "email": sender})

        if valid_sender.status_code == 200:
            data = json.loads(valid_sender.content)
            if data["is_enabled"]:
                self.connection = True
                return True
        else:
            if not self.fail_silently:
                raise

    def send_messages(self, email_messages):
        if not email_messages:
            return

        num_sent = 0
        for message in email_messages:
            self.open(message.from_email)
            if not self.connection:
                return

            sent = self._send(message)

            if sent:
                num_sent += 1

        return num_sent

    def _send(self, message):
        if not message.recipients():
            return False

        self.sender = sanitize_address(message.from_email, message.encoding)
        recipients_list = [sanitize_address(addr, message.encoding)
            for addr in message.recipients()]
        from email.utils import parseaddr
        self.recipients = [{"email": e, "name": n} for n,e in [
            parseaddr(r) for r in recipients_list]]

        self.msg_dict = self._build_standard_message_dict(message)

        if getattr(message, "alternative_subtype", None):
            if message.alternative_subtype == "mandrill":
                self._build_advanced_message_dict(message)
                if message.alternatives:
                    self._add_alternatives(message)

        djrill_it = requests.post(self.api_action, data=json.dumps({
            "key": self.api_key,
            "message": self.msg_dict
        }))

        if djrill_it.status_code != 200:
            if not self.fail_silently:
                raise
            return False
        return True

    def _build_standard_message_dict(self, message):
        """
        Build standard message dict.

        Builds the standard dict that Django's send_mail and send_mass_mail
        use by default. Standard text email messages sent through Django will
        still work through Mandrill.
        """
        return {
            "text": message.body,
            "subject": message.subject,
            "from_email": self.sender,
            "to": self.recipients
        }

    def _build_advanced_message_dict(self, message):
        """
        Builds advanced message dict and attaches any accepted extra headers.
        """
        self.msg_dict.update({
            "from_name": message.from_name,
            "tags": message.tags,
            "track_opens": message.track_opens,
        })

        if message.extra_headers:
            accepted_headers = {}

            for k in message.extra_headers.keys():
                if k.startswith("X-") or k == "Reply-To":
                    accepted_headers.update(
                        {"%s" % k: message.extra_headers[k]})
            self.msg_dict.update({"headers": accepted_headers})

    def _add_alternatives(self, message):
        """
        There can be only one! ... alternative attachment.

        Since mandrill does not accept image attachments or anything other
        than HTML, the assumption is the only thing you are attaching is
        the HTML output for your email.
        """
        if len(message.alternatives) > 1:
            raise ImproperlyConfigured(
                "Mandrill only accepts plain text and html emails. Please "
                "check the alternatives you have attached to your message.")

        self.msg_dict.update({
            "html": message.alternatives[0][0],
            "track_clicks": message.track_clicks
        })



from setuptools import setup

setup(
    name="djrill",
    version="0.1.1",
    description='Django email backend for Mandrill.',
    long_description='Email backend and new message class to send emails through the Mandrill email service.',
    keywords="django, mailchimp, mandrill, email, email backend",
    author="Kenneth Love <kenneth@brack3t.com>, Chris Jones <chris@brack3t.com>",
    author_email="kenneth@brack3t.com",
    url="https://github.com/brack3t/Djrill/",
    license="BSD",
    packages=["djrill"],
    zip_safe=False,
    install_requires=["requests", "django"],
    include_package_data=True,
    classifiers=[
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Framework :: Django",
        "Environment :: Web Environment",
    ],
)



from django.contrib import admin

from djrill.views import (DjrillIndexView, DjrillSendersListView,
                          DjrillDisableSenderView, DjrillVerifySenderView,
                          DjrillAddSenderView, DjrillTagListView,
                          DjrillUrlListView)

admin.site.register_view("djrill/senders/", DjrillSendersListView.as_view(),
    "djrill_senders", "senders")
admin.site.register_view("djrill/status/", DjrillIndexView.as_view(),
    "djrill_status", "status")
admin.site.register_view("djrill/tags/", DjrillTagListView.as_view(),
    "djrill_tags", "tags")
admin.site.register_view("djrill/urls/", DjrillUrlListView.as_view(),
    "djrill_urls", "urls")

admin.site.register_url("djrill/disable/sender/",
    DjrillDisableSenderView.as_view(), "djrill_disable_sender")
admin.site.register_url("djrill/verify/sender/",
    DjrillVerifySenderView.as_view(), "djrill_verify_sender")
admin.site.register_url("djrill/add/sender/",
    DjrillAddSenderView.as_view(), "djrill_add_sender")



from django.contrib.admin.sites import AdminSite
from django.utils.text import capfirst

VERSION = (0, 1, 0)
__version__ = '.'.join([str(x) for x in VERSION])


class DjrillAdminSite(AdminSite):
    index_template = "djrill/index.html"
    custom_views = []
    custom_urls = []

    def register_view(self, path, view, name, display_name=None):
        """Add a custom admin view.

        * `path` is the path in the admin where the view will live, e.g.
            http://example.com/admin/somepath
        * `view` is any view function you can imagine.
        * `name` is an optional pretty name for the list of custom views. If
            empty, we'll guess based on view.__name__.
        """
        self.custom_views.append((path, view, name, display_name))

    def register_url(self, path, view, name):
        self.custom_urls.append((path, view, name))

    def get_urls(self):
        """Add our custom views to the admin urlconf."""
        urls = super(DjrillAdminSite, self).get_urls()
        from django.conf.urls.defaults import patterns, url
        for path, view, name, display_name in self.custom_views:
            urls += patterns('',
                url(r'^%s$' % path, self.admin_view(view), name=name),
            )
        for path, view, name in self.custom_urls:
            urls += patterns('',
                url(r'^%s$' % path, self.admin_view(view), name=name),
            )

        return urls

    def index(self, request, extra_context=None):
        """Make sure our list of custom views is on the index page."""
        if not extra_context:
            extra_context = {}
        custom_list = [(path, display_name if display_name else
            capfirst(view.__name__)) for path, view, name, display_name in
            self.custom_views]
        # Sort views alphabetically.
        custom_list.sort(key=lambda x: x[1])
        extra_context.update({
            'custom_list': custom_list
        })
        return super(DjrillAdminSite, self).index(request, extra_context)
